file_id = "ARDUINO_IO_GET"
log_msg("start..",file_id)

--[[
**************************************************************************************
現在の Arduino デバイスの I/O 値を取得する

リターン時に返されるパラメータ
---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------
PORT_0			Arduino port0(8bit)の現在の値(16進数2桁)				"FC"
PORT_1			Arduino port1(8bit)の現在の値(16進数2桁)				"00"
..
PORT_n			Arduino portn(8bit)の現在の値(16進数2桁)				"00"

ATmega328 を使用した Arduino ボードの場合には、 digital I/O port は 0,1 の2 つで
それぞれ、port0:PORTD, port1:PORTB(bit#0-bit#5) + (PORTC bit#0-#1)に対応しています。
詳しくは、Arduino IDE に同梱されている pins_arduino.h ファイルを参照して下さい。
**************************************************************************************
]]

----------------------------------------------------------------------
-- Arduino digital I/O ポート取得 QUERY 送信と REPLY 受信
----------------------------------------------------------------------
local com = "COM4"
local sysex_start = "F0"
local sysex_end = "F7"
local firmata_request = "61"
local firmata_reply = "62"
local send_data = sysex_start .. firmata_request .. sysex_end

----------------------------------------------------------------------
-- 排他制御開始
-- 同じ Arduino デバイスに対して、I/O PORT QUERY コマンドが複数同時実行される場合に備えています。
-- 別のArduino デバイスを操作する場合や、同じ Arduino デバイスに対して異なる FIRMATA REPLY
-- コマンドバイト値を取得する場合には排他制御の必要はありません
----------------------------------------------------------------------
local cstat,handle = critical_session_enter("FirmataQueyOP" .. com,5000);
if not cstat then
	log_msg("critical_session_enter() failed",file_id)
	error()
end

----------------------------------------------------------------------
-- リクエストフレーム送信とリプライ受信
----------------------------------------------------------------------
local stat,rdata = serial_write(com,send_data,firmata_reply)

----------------------------------------------------------------------
-- 排他制御終了
----------------------------------------------------------------------
cstat = critical_session_leave(handle)
if not cstat then
	log_msg("critical_session_leave() failed",file_id)
	error()
end
if not stat then
	log_msg("serial_write() failed",file_id)
	error()
end

----------------------------------------------------------------------
-- 受信した FIRMATA フレームの内容をリターンパラメータに格納
----------------------------------------------------------------------
local data = hex72_to_tbl(string.sub(rdata,5,-3),2)
if data ~= nil then
	for key,val in ipairs(data) do
	 	if not script_result(g_taskid,"PORT_" .. tostring(key - 1),bit_tohex(val,2)) then error() end
	end
end

