
file_id = "SERIAL_FIRMATA"

--[[

******************************************************************************
* イベントハンドラスクリプト実行時間について                                 *
******************************************************************************

一つのスクリプトの実行は長くても数秒以内で必ず終了するようにしてください。
処理に時間がかかると、イベント処理の終了を待つサーバー側でタイムアウトが発生します。

また、同時実行可能なスクリプトの数に制限があるため、他のスクリプトの実行開始が
待たされる原因にもなります。

頻繁には発生しないイベントで、処理時間がかかるスクリプトを実行したい場合は
スクリプトを別に作成して、このイベントハンドラ中から script_fork_exec() を使用して
別スレッドで実行することを検討してください。

******************************************************************************


SERIAL_RAW スクリプト起動時に渡される追加パラメータ

---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------
COMPort			イベントを送信したシリアルデバイスの COMポート名		"COM3"

FIRMATA_DATA	COM ポートから読み込んだ FIRMATA プロトコルパケットのバイナリデータ。
				(16進数、可変長)										"0102030A0B"
---------------------------------------------------------------------------------

]]

-------------------------------------
-- FIRMATA イベントデータをログに出力
-------------------------------------
local str = ""
for key,val in pairs(g_params) do
	str = str .. key .. " = " .. val .. " "
end
log_msg(str,file_id);

---------------------------------------------------------------
-- SYSEX コマンド "DIGITAL_PORTS_MESSAGE" を受信した場合の処理
---------------------------------------------------------------
if (string.sub(g_params["FIRMATA_DATA"],1,4) == "F064") then 

	local msg = "{"	
	local data = hex72_to_tbl(string.sub(g_params["FIRMATA_DATA"],5,-3),2)
	if data ~= nil then
		for key,val in ipairs(data) do
			if key > 1 then msg = msg .. ',' end
 			msg = msg .. '"PORT_' .. tostring(key - 1) .. '":"' .. bit_tohex(val,2) .. '"'
		end
	end
	msg = msg .. "}"

	log_msg("uplink message = " .. msg,file_id);
	local nstat,rdata = tcp_send_recv_data("localhost",9080,msg,2,2)
	if not nstat then error() end
end

